package com.tzld.piaoquan.wecom.service.Impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.wecom.common.exception.CustomizeException;
import com.tzld.piaoquan.wecom.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.wecom.model.po.MessageAttachment;
import com.tzld.piaoquan.wecom.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.wecom.model.vo.MiniprogramVo;
import com.tzld.piaoquan.wecom.service.MessageAttachmentService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.HttpPoolClient;
import com.tzld.piaoquan.wecom.utils.MessageUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.*;

import static com.tzld.piaoquan.wecom.common.constant.MessageConstant.appid;
import static com.tzld.piaoquan.wecom.common.constant.OtherServerURL.POST_VIDEO_DETAIL_URL;
import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.GUARANTEED_MINIPROGRAM_KEY;
import static com.tzld.piaoquan.wecom.common.enums.ExceptionCodeEnum.PARAMS_ERROR;


@Service
public class MessageAttachmentServiceImpl implements MessageAttachmentService {

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(30000, 30000, 2000, 5000, 5, 30000);

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public void addMiniprogram(List<MessageAttachment> messageAttachmentList) {
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria()
                    .andAppidEqualTo(messageAttachment.getAppid())
                    .andTypeEqualTo(messageAttachment.getType())
                    .andMiniprogramVideoIdEqualTo(messageAttachment.getMiniprogramVideoId());
            List<MessageAttachment> messageAttachments = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachments)) {
                messageAttachmentMapper.insert(messageAttachment);
                continue;
            }
            messageAttachment.setId(messageAttachments.get(0).getId());
            messageAttachmentMapper.updateByPrimaryKeySelective(messageAttachment);
        }
    }

    @Override
    public void createGuaranteedMiniprogram(List<MiniprogramVo> miniprogramVoList) {
        List<MessageAttachment> messageAttachmentList = new ArrayList<>();
        List<Long> videoList = new ArrayList<>();
        for (MiniprogramVo miniprogramVo : miniprogramVoList) {
            if (StringUtils.isEmpty(miniprogramVo.getPage())) {
                throw new CustomizeException(PARAMS_ERROR);
            }
            if (StringUtils.isEmpty(miniprogramVo.getTitle())) {
                throw new CustomizeException(PARAMS_ERROR);
            }
            Long videoId = MessageUtil.getVideoId(miniprogramVo.getPage());
            if (videoId == null) {
                throw new CustomizeException(PARAMS_ERROR);
            }
            MessageAttachment messageAttachment = new MessageAttachment();
            BeanUtils.copyProperties(miniprogramVo, messageAttachment);
            messageAttachment.setMiniprogramVideoId(videoId);
            messageAttachment.setType(MessageAttachmentTypeEnum.MINIPROGRAM.getType());
            messageAttachmentList.add(messageAttachment);
            videoList.add(videoId);
        }
        Map<Long, String> coverMap = getCover(videoList);
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            String cover = coverMap.get(messageAttachment.getMiniprogramVideoId());
            if (StringUtils.isEmpty(cover)) {
                throw new RuntimeException("获取封面异常");
            }
            messageAttachment.setCover(cover);
            messageAttachment.setAppid(appid);
        }
        addMiniprogram(messageAttachmentList);
        for (Long video : videoList) {
            redisTemplate.opsForList().rightPush(GUARANTEED_MINIPROGRAM_KEY, video);
        }
    }

    public Map<Long, String> getCover(List<Long> videoIdList) {
        try {
            JSONObject params = new JSONObject();
            params.put("videoIdList", videoIdList);
            String post = httpPoolClientDefault.post(POST_VIDEO_DETAIL_URL, params.toJSONString());
            JSONObject res = JSONObject.parseObject(post);
            JSONArray data = res.getJSONArray("data");
            Map<Long, String> map = new HashMap<>();
            for (int i = 0; i < data.size(); i++) {
                JSONObject jsonObject = data.getJSONObject(i);
                Long videoId = jsonObject.getLong("id");
                JSONObject coverImg = jsonObject.getJSONObject("coverImg");
                String coverImgPath = coverImg.getString("coverImgPath");
                map.put(videoId, coverImgPath);
            }
            return map;
        } catch (IOException e) {
            throw new RuntimeException("获取封面异常");
        }
    }
}
